import "@picocss/pico/css/pico.min.css";
import "./style.css";
import { setupRequestPage } from "./routes/request.ts";
import { setupSendPage } from "./routes/send.ts";
import { setupReceivePage } from "./routes/receive.ts";
import {
  exportKeys,
  importAndSaveKeys,
  retrieveOrGenerateKeyPair,
} from "./utils/crypto.ts";

const updateKeysInModal = () => {
  console.log("help");
  const publicKeyElement = document.getElementById("ecdhPublicKey");
  const privateKeyElement = document.getElementById("ecdhPrivateKey");

  const publicKey = localStorage.getItem("ecdhPublic");
  const privateKey = localStorage.getItem("ecdhPrivate");

  if (!publicKeyElement || !privateKeyElement) {
    throw new Error("Key manager elements not found");
  }

  if (publicKey && privateKey) {
    publicKeyElement.textContent = publicKey;
    privateKeyElement.textContent = privateKey;
  }
};

const start = () => {
  const fragmentData = new URLSearchParams(window.location.hash.slice(1));
  const appElement = document.querySelector<HTMLElement>("#app");
  const keyManagerEl = document.getElementById("openKeyManager");
  const keyManagerModal = document.getElementById("keyManagerModal");
  const closeKeyManager = document.getElementById("closeKeyManager");

  if (!appElement) {
    throw new Error("No app element found");
  }

  if (!keyManagerEl || !keyManagerModal || !closeKeyManager) {
    throw new Error("No key manager element found");
  }

  retrieveOrGenerateKeyPair();
  keyManagerEl.addEventListener("click", () => {
    updateKeysInModal();
    keyManagerModal.setAttribute("open", "");
  });

  closeKeyManager.addEventListener("click", () => {
    keyManagerModal.removeAttribute("open");
  });

  // If the URL contains the 'p', 'iv', and 'm' parameters, then the receiver page is set up
  if (
    fragmentData.has("p") &&
    fragmentData.has("iv") &&
    fragmentData.has("m")
  ) {
    setupReceivePage(appElement, {
      p: decodeURIComponent(fragmentData.get("p")!),
      iv: decodeURIComponent(fragmentData.get("iv")!),
      m: decodeURIComponent(fragmentData.get("m")!),
    });
    return;
  }

  // If the URL contains the 'p' parameter, then the sender page is set up
  if (fragmentData.has("p")) {
    setupSendPage(appElement, fragmentData.get("p")!);
    return;
  }

  // Otherwise, the request page is set up
  setupRequestPage(appElement);
};

const setupKeyManager = async () => {
  const exportKeyEl = document.getElementById("exportKeypair");
  const importKeyEl = document.getElementById("importKeypair");

  if (!exportKeyEl || !importKeyEl) {
    throw new Error("Key management elements not found");
  }

  exportKeyEl.addEventListener("click", exportKeys);
  importKeyEl.addEventListener("click", () => {
    const fileInput = document.createElement("input");
    fileInput.type = "file";
    fileInput.style.display = "none";
    document.body.appendChild(fileInput);

    fileInput.click();

    fileInput.addEventListener("change", async (e) => {
      const file = (e.target as HTMLInputElement).files?.[0];
      if (!file) return;

      const reader = new FileReader();
      reader.onload = async (e) => {
        const keys = JSON.parse(e.target?.result as string);
        await importAndSaveKeys(keys);

        document.body.removeChild(fileInput);
        alert("Successfully imported keypair");
      };
      reader.readAsText(file);
    });
  });
};

start();
setupKeyManager();
