import { retrieveOrGenerateKeyPair } from "../utils/crypto.ts";

const URL_INPUT_ID = "request-url";
const COPY_BUTTON_ID = "copy-button";
const TEMPLATE = `
      <details open>
        <summary>How To Use:</summary>
        <ol>
        <li>Send the link below to someone you want to get a secret from.</li>
        <li>They add their secret and share the URL the app generates.</li>
        <li>Only your browser can open the URL and decrypt the secret.</li>
        </ol>
      </details>

      <form>
        <input
          type="text"
          name="request-url"
          id="${URL_INPUT_ID}"
          placeholder=""
          aria-label="Request URL"
          required
        />
        <button aria-label="Copy to Clipboard" id="${COPY_BUTTON_ID}">Copy to Clipboard</button>
      </form>
    `;

export async function setupRequestPage(element: HTMLElement) {
  element.innerHTML = TEMPLATE;

  const url = await generateUrl();
  const input = document.getElementById(URL_INPUT_ID) as HTMLInputElement;
  input.value = url.toString();

  const copyButton = document.getElementById(
    COPY_BUTTON_ID
  ) as HTMLButtonElement;
  copyButton.addEventListener("click", copyToClipboard);
}

async function copyToClipboard(event: Event) {
  event.preventDefault();
  const input = document.getElementById(URL_INPUT_ID) as HTMLInputElement;
  await navigator.clipboard.writeText(input.value);
  alert("Copied to clipboard!");
}

async function generateUrl(): Promise<URL> {
  const { publicKey: ecdhPublic } = await retrieveOrGenerateKeyPair();

  // Generate URL with public key as the 'p' search parameter
  const ecdhPublicJwk = await window.crypto.subtle.exportKey("jwk", ecdhPublic);
  const url = new URL(window.location.toString());
  url.search = "";
  url.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}`;

  // Return the generated URL
  return url;
}
